-- Mocksamdi
-- © 2020 CC0 1.0 Universal (CC0 1.0) Public Domain Dedication. https://creativecommons.org/publicdomain/zero/1.0/

-- ===================================================
-- Constants
-- ===================================================

local ADDON_NAME = "Mocksamdi";
local ADDON_VERSION = "0.2";
local ADDON_DEBUG = false;

local E_ADDON_LOADED = "ADDON_LOADED";  -- fires when saved variables are loaded
local E_PLAYER_DEAD = "PLAYER_DEAD";  -- fires whern player dies

local ALL_EVENTS = {
  E_ADDON_LOADED,
  E_PLAYER_DEAD
}

local SLASH_REG = "MOCKSAMDI";
local SLASH_CMD1 = "/msa";
local SLASH_CMD2 = "/mocksamdi";

local GENERAL_SOUNDS = {
	94633,  -- what ya doin here man? not your time
	98423,  -- mighty hero
	98428,  -- dangerous games
	98429,  -- i see dead people
	98430,  -- want a zombie drink?
	98431,  -- everyone comes to me
	109291, -- still owe me
	109293, -- got in over your head
	109294, -- did not expect to see you
	109295, -- poor little thing
	109296, -- impressive
	109297  -- quality time
};

WARIROR_SOUND = 109308; -- 1
PALADIN_SOUND = 109304; -- 2
HUNTER_SOUND = 109301; -- 3
ROGUE_SOUND = 109305; -- 4
PRIEST_SOUND = 109304; -- 5
DEATH_KNIGHT_SOUND = 109298; -- 6
SHAMEN_SOUND = 109306; -- 7
MAGE_SOUND = 109302; -- 8
WARLOCK_SOUND = 109307; -- 9
MONK_SOUND = 109303; -- 10
DRUID_SOUND = 109300; -- 11
DEMON_HUNTER_SOUND = 109299; -- 12

-- ===================================================
-- Variables
-- ===================================================

local frame = CreateFrame("FRAME"); -- respond to events
local classType = 0;
local lastPlayedSoundIndex = 0;
local configPanel = CreateFrame("Frame"); -- config panel

-- ===================================================
-- Functions
-- ===================================================

-- Debug Function
local printDebug = function (s)
	if ADDON_DEBUG then
		print("|cff00ccffMS DEBUG: " .. s .. "|r");
	end
end

-- Get class specific sound
local getClassSound = function (classId)
	if classId == 1 then
		return WARIROR_SOUND;
	elseif classId == 2 then
		return PALADIN_SOUND;
	elseif classId == 3 then
		return HUNTER_SOUND;
	elseif classId == 4 then
		return ROGUE_SOUND;
	elseif classId == 5 then
		return PRIEST_SOUND;
	elseif classId == 6 then
		return DEATH_KNIGHT_SOUND;
	elseif classId == 7 then
		return SHAMEN_SOUND;
	elseif classId == 8 then
		return MAGE_SOUND;
	elseif classId == 9 then
		return WARLOCK_SOUND;
	elseif classId == 10 then
		return MONK_SOUND;
	elseif classId == 11 then
		return DRUID_SOUND;
	elseif classId == 12 then
		return DEMON_HUNTER_SOUND;
	end

	-- fallback
	printDebug("Class was " .. classId);
	return 94633;
end

-- Play mock sounds
local playMockSound = function ()
	local soundCount = table.getn(GENERAL_SOUNDS);
	local totalSoundCount = soundCount;

	-- load class and add the class specific one to the rotation
	localizedClass, englishClass, classIndex = UnitClass("player");
	if classIndex > 0 then
		totalSoundCount = soundCount + 1;
	end

	local soundIndex = math.random(1, totalSoundCount);

	-- lazy dedupe (you should probably make this better and store an array)
	if soundIndex == lastPlayedSoundIndex then
		printDebug("Dupe sound " .. soundIndex .. ". Picking another");
		soundIndex = soundIndex + 1;
	end
	lastPlayedSoundIndex = soundIndex; -- store the new played sound

	printDebug("Picked random number " .. soundIndex .. " of " .. totalSoundCount);

	if soundIndex > soundCount then
		local classSound = getClassSound(classIndex)
		printDebug("Playing class sound " .. classSound);
		PlaySound(classSound);
	else
		printDebug("Playing sound " .. GENERAL_SOUNDS[soundIndex]);
		PlaySound(GENERAL_SOUNDS[soundIndex]);
	end
end

-- slash commands
local commands = function (msg, editbox)
  -- pattern matching that skips leading whitespace and whitespace between cmd and args
  -- any whitespace at end of args is retained
	local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)");

	printDebug("Slash command fired");

	InterfaceOptionsFrame_Show();
	InterfaceOptionsFrame_OpenToCategory(ADDON_NAME);
	InterfaceOptionsFrame_OpenToCategory(ADDON_NAME); -- handle open bug
end

-- Init Config
local initConfig = function ()
	local placement = -10;
	configPanel.name = ADDON_NAME;

	configPanel.text = configPanel:CreateFontString(nil, "ARTWORK");
	configPanel.text:SetFont("Fonts\\FRIZQT__.TTF", 20, "OUTLINE");
	configPanel.text:SetPoint("TOPLEFT", 10, placement);
	configPanel.text:SetText(ADDON_NAME .. " Configuration");
	placement = placement - 25;

	-- MocksamdiShowStartup
	local showStartupCB = CreateFrame("CheckButton", "showStartupCB_GlobalName", configPanel, "OptionsCheckButtonTemplate");
	showStartupCB:SetPoint("TOPLEFT", 10, placement);
	showStartupCB_GlobalNameText:SetText("Show addon startup text");
	showStartupCB:SetChecked(MocksamdiShowStartup);
	showStartupCB:SetScript("OnClick",
		function()
			MocksamdiShowStartup = showStartupCB:GetChecked();
		end
	);
	placement = placement - 25;

	-- MocksamdiShowText
	-- local showTextCB = CreateFrame("CheckButton", "showTextCB_GlobalName", configPanel, "OptionsCheckButtonTemplate");
	-- showTextCB:SetPoint("TOPLEFT", 10, placement);
	-- showTextCB_GlobalNameText:SetText("Show dialog text");
	-- showTextCB:SetChecked(MocksamdiShowText);
	-- showTextCB:SetScript("OnClick",
	-- 	function()
	-- 		MocksamdiShowText = showTextCB:GetChecked();
	-- 	end
	-- );
	-- placement = placement - 25;

	-- MocksamdiPlayAudio
	local playAudioCB = CreateFrame("CheckButton", "playAudioCB_GlobalName", configPanel, "OptionsCheckButtonTemplate");
	playAudioCB:SetPoint("TOPLEFT", 10, placement);
	playAudioCB_GlobalNameText:SetText("Play audio on death (If you disable this the addon will do nothing)");
	playAudioCB:SetChecked(MocksamdiPlayAudio);
	playAudioCB:SetScript("OnClick",
		function()
	 		MocksamdiPlayAudio = playAudioCB:GetChecked();
	 	end
	);
	placement = placement - 25;

	-- MocksamdiShowImage
	-- local showImageCB = CreateFrame("CheckButton", "showImageCB_GlobalName", configPanel, "OptionsCheckButtonTemplate");
	-- showImageCB:SetPoint("TOPLEFT", 10, placement);
	-- showImageCB_GlobalNameText:SetText("Show an image of Bwonsamdi on death");
	-- showImageCB:SetChecked(MocksamdiShowImage);
	-- showImageCB:SetScript("OnClick",
	-- 	function()
	-- 		MocksamdiShowImage = showImageCB:GetChecked();
	-- 	end
	-- );
	-- placement = placement - 25;

	-- Append to options panel
	InterfaceOptions_AddCategory(configPanel);
end

-- ===================================================
-- Event Checker
-- ===================================================

-- register events
for i=1, table.getn(ALL_EVENTS) do
	printDebug("Register Event: " .. ALL_EVENTS[i]);
	frame:RegisterEvent(ALL_EVENTS[i]);
end

frame:SetScript("OnEvent",
	function (self, event, arg1)
		-- Debug all events
		-- printDebug("Event: " .. event .. " " .. arg1);

		-- Startup
		if event == E_ADDON_LOADED and arg1 == ADDON_NAME then
			-- set defaults if needed
			if MocksamdiShowStartup == nil then MocksamdiShowStartup = true end
			if MocksamdiShowText == nil then MocksamdiShowText = true end
			if MocksamdiShowImage == nil then MocksamdiShowImage = true end
			if MocksamdiPlayAudio == nil then MocksamdiPlayAudio = true end

			-- debug variable values
			printDebug("Saved Vars:")
			printDebug("- MocksamdiShowStartup: " .. (MocksamdiShowStartup and 'true' or 'false'));
			printDebug("- MocksamdiShowText: " .. (MocksamdiShowText and 'true' or 'false'));
			printDebug("- MocksamdiShowImage: " .. (MocksamdiShowImage and 'true' or 'false'));
			printDebug("- MocksamdiPlayAudio: " .. (MocksamdiPlayAudio and 'true' or 'false'));

			initConfig();

			-- Check show startup message
			if MocksamdiShowStartup == true then
				MocksamdiShowStartup = true;
				print("Loaded |cff00E5EE" .. ADDON_NAME .. "|r at version |cffffcc00" .. ADDON_VERSION .. "|r")
			end
		end

		-- Check if died
		if event == E_PLAYER_DEAD then
			printDebug("Dead");
			if MocksamdiPlayAudio then
				playMockSound();
			end
		end
	end);

-- ===================================================
-- Slash Commands
-- ===================================================

SLASH_MOCKSAMDI1, SLASH_MOCKSAMDI2 = SLASH_CMD1, SLASH_CMD2;
SlashCmdList[SLASH_REG] = commands;